/*+------------------------------------------------------------------------+
  | Copyright (c) 2003,2004 by SAS Institute Inc., Cary, NC 27513
  | Name:     SPM Table and View Generation and Registry
  | Macro:    SPMETL autocall macro
  | Product:  SAS Strategic Performance Management
  | Language: SAS
  | Author:   SASTZH
  | Purpose:
  |
  | SAS Macro that generates all the SQL views and placeholder tables
  | associated with the following processes:
  |
  | PM History Migration
  | Load PM Metrics Data
  | Reporting / Consulting Views
  |
  | Then registers those that need to be surfaced via ETL Studio
  |
  +------------------------------------------------------------------------+*/
%macro spmetl(register=Y);

%let register=%upcase(&register);

   %macro regspm(libref=,
                tablename=,
                omr_debug=N,
                ddsTree=,
                ddsTreeName=,
                ddsVersion=PM SAS Supplied Model - v1.0
              );

     %nrstr(%regMdata)
                (sesetl_serverContext=&iomserver,
                 sesetl_sasLibrary=&libref,
                 sesetl_physicalTable=&tableName,
                 sesetl_ddsTree=&ddsTree,
                 sesetl_ddsVersion=&ddsVersion,
                 sesetl_tablePW=,
                 sesetl_RegIfEmpty=Y,
                 sesetl_altETLStudioTreeNm=SAS Data Integration Studio Custom Tree,
                 sesetl_altEtlStudioGroupNm=Data Integration Studio Groups,
                 sesetl_ddsTreeName=&ddsTreeName,
                 sesetl_ddsValidTreeTypes=S:Staging Tables @
                                          T:Status Tables @
                                          D:Dimension Tables @
                                          R:Reference Tables @
                                          F:Fact Tables @
                                          V:Reference Views,
                 sesetl_cacheGroupNm=,
                 sesetl_ChkDepRepos=Y,
                 sesetl_summaryRoleDs=,
                 sesetl_OMRTableObjName=,
                 sesetl_runscd=,
                 sesetl_businesskey=,
                 sesetl_surrogatekey=,
                 sesetl_security=
                ) ;
   %mend ;

 /*+------------------------------------------------------------------------+
  | Allocate SASSDM library
  +------------------------------------------------------------------------+*/

  options metarepository="&SlnRepos";

  %bldLibNm(sesetl_sasLibRef=SASSDM,
            sesetl_OMRSASLibraryObjName=,
            sesetl_ChkDepRepos=Y,
            sesetl_useMLE=N
           ) ;

  %checkLib(sesetl_libref=SASSDM) ;


/*+------------------------------------------------------------------------+
  | Allocate SPMDS library
  +------------------------------------------------------------------------+*/
  options metarepository="&pmrepos";

  %bldLibNm(sesetl_sasLibRef=SPMDS,
            sesetl_OMRSASLibraryObjName=,
            sesetl_ChkDepRepos=Y,
            sesetl_useMLE=N
           ) ;

  %checkLib(sesetl_libref=SPMDS);

/*+------------------------------------------------------------------------+
  | Allocate SPMData library
  +------------------------------------------------------------------------+*/
   options metarepository="&pmrepos";

   %bldLibNm(sesetl_sasLibRef=SPMData,
             sesetl_OMRSASLibraryObjName=,
             sesetl_ChkDepRepos=Y,
             sesetl_useMLE=N
            ) ;
   %let iomserver=&sesetl_serverContext;

   %checkLib(sesetl_libref=SPMData);



proc sql;

/*+------------------------------------------------------------------------+
  | The following views are specifically for use during the History Data
  | Migration process from SPM 1.4 to SPM 2.0.  Even though these views will
  | reside in the SPMDATA library, they will not be registered in OMR;
  | therefore, they will not appear in SAS/ETL Studio, etc.
  |
  +------------------------------------------------------------------------+*/
/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.MIGRATION_COL_MAP_VIEW
  | Purpose:
  |
  | View that provides a mapping between SPM 2.0 COLUMN_IDs and SPM 1.4
  | CKEYs. In SPM 2.0, COLUMN_IDs are unique, but in SPM 1.4 columns
  | are unique only when combining the 1.4 ENVKEY and CKEYs. The following
  | columns are contained in this view:
  |
  | Name             Type Length   Format   Label
  |
  | COLUMN_ID        C     32      $32.     SPM2.0 Column Id
  | CKEY             C     32      $32.     SPM1.4 Column Id
  | ENVKEY           C     32      $32.     SPM1.4 Environment Id
  |
  | NOTE: This view is designed only for use in SPM HISTORY DATA MIGRATION.
  +------------------------------------------------------------------------+*/

create view SPMDATA.MIGRATION_COL_MAP_VIEW as select

   columndef.Id as COLUMN_ID label='SPM2.0 Column Id',
   columndef.V14Key as CKEY  label='SPM1.4 Column Id',
   project.V14Key as ENVKEY  label='SPM1.4 Environment Id'

from

   SPMDS.label,
   SPMDS.columndef,
   SPMDS.project

where

   columndef.TemplateId = project.TemplateId and
   label.Id = columndef.Id and
   (columndef.V14Key Is Not Null) and
   (project.V14Key Is Not Null);

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.MIGRATION_PROJECT_VIEW
  | Purpose:
  |
  | View that provides a mapping between SPM 2.0 Projects and SPM 1.4
  | Environments. The following columns are contained in this view:
  |
  | Name                Type  Length  Format   Label
  |
  | PROJECT_ID          C     32      $32.     SPM2.0 Project Id
  | TIME_DIMENSION_ID   N      8      $11.     SPM2.0 Time Dimension Id
  | TIME_HIERARCHY_ID   N      8      $11.     SPM2.0 Time Hierarchy Id
  | ENVKEY              C     32      $32.     SPM1.4 Environment Id
  |
  | NOTE: This view is designed only for use in SPM HISTORY DATA MIGRATION.
  +------------------------------------------------------------------------+*/

create view SPMDATA.MIGRATION_PROJECT_VIEW as select

   project.Id as PROJECT_ID label='SPM2.0 Project Id',
   project.timeDimensionId as TIME_DIMENSION_ID
      label='SPM2.0 Time Dimension Id',
   project.timeHierarchyId as TIME_HIERARCHY_ID
      label='SPM2.0 Time Hierarchy Id',
   project.V14Key as ENVKEY  label='SPM1.4 Environment Id'

from

   SPMDS.project

where

   project.V14Key Is not Null;

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.MIGRATION_SC_EL_ENV_MAP_VIEW
  | Purpose:
  |
  | View that provides a mapping between SPM 2.0 Elements and SPM 1.4
  | Elements. In SPM 2.0, ELEMENT_IDs are unique, but in SPM 1.4 Elements
  | are unique only when combining the 1.4 Environment Id (ENVKEY),
  | 1.4 Scorecard Id (TKEY) and Element Id (RKEY). The following
  | columns are contained in this view:
  |
  | Name             Type Length   Format   Label
  |
  | ELEMENT_ID       C     32      $32.     SPM2.0 Element Id
  | RKEY             C     32      $32.     SPM1.4 Element Id
  | TKEY             C     32      $32.     SPM1.4 ScoreCard Id
  | ENVKEY           C     32      $32.     SPM1.4 Environment Id
  |
  | NOTE: This view is designed only for use in SPM HISTORY DATA MIGRATION.
  +------------------------------------------------------------------------+*/

create view SPMDATA.MIGRATION_SC_EL_ENV_MAP_VIEW as select

   tempview.V20_EL_ID as ELEMENT_ID label='SPM2.0 Element Id',
   tempview.V14_EL_ID as RKEY   label='SPM1.4 Element Id',
   tempview.V14_SC_ID as TKEY   label='SPM1.4 ScoreCard Id',
   project.V14Key as ENVKEY  label='SPM1.4 Environment Id'

from

   SPMDS.project as c,
   (
      select
         a.Id as V20_EL_ID,
         b.ProjectId as V20_PROJ_ID,
         a.V14Key as V14_EL_ID,
         b.V14Key as V14_SC_ID

      from

         SPMDS.element as a,
         SPMDS.scorecard as b

      where

         a.V14Key Is Not Null and (a.ContainerId=b.Id)

   ) as tempview

where

   tempview.V20_PROJ_ID=c.Id;


/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.MIGRATION_TIME_VIEW
  | Purpose:
  |
  | View that contains the following information for all dimension values
  | in all dimensions of type TIME in SASSDM (Common Data Mart):
  |
  | Name              Type  Length   Format      Label
  |
  | TIME_DIMENSION_ID  N      8      11.          Time Dimension Id
  | PERIOD_ID          N      8      11.          Time Period Id
  | PERIOD_CODE        C     32      $32.         Time Period Code
  | START_DATE         N      8      YYMMDDN8.    Start Date
  | END_DATE           N      8      YYMMDDN8     End Date
  | PERIOD_TYPE        C    255      $255.        Time Period Type
  | VALID_FROM_DTTM    N      8      DATETIME20.1 Valid From Datetime
  | VALID_TO_DTTM      N      8      DATETIME20.1 Valid To Datetime
  |
  +------------------------------------------------------------------------+*/

create view SPMDATA.MIGRATION_TIME_VIEW as select DISTINCT

   tempvue.dimension_rk as TIME_DIMENSION_ID label='Time Dimension Id',
   tempvue.member_rk as PERIOD_ID label='Time Period Id',
   e.member_cd as PERIOD_CODE label='Time Period Code',
   max (case
        when property_cd = 'StartDate' then
            input(property_value,yymmdd10.)
        else .
        end) as START_DATE label='Start Date' format=yymmddn8.,
   max (case
      when property_cd = 'EndDate' then
           input(property_value,yymmdd10.)
      else .
      end) as END_DATE label='End Date' format=yymmddn8.,
   max  (case
      when property_cd = 'Level' then
           property_value
      else ' '
      end ) as PERIOD_TYPE label='Time Period Type',

   e.VALID_FROM_DTTM label='Valid From Datetime',
   e.VALID_TO_DTTM label='Valid To Datetime'

from sassdm.member as e,
   (
       select DISTINCT

          c.member_rk,
          a.dimension_rk,
          a.dim_internal_rk,
          b.property_cd,
          c.property_value,
          c.valid_from_dttm,
          c.valid_to_dttm

       from

          sassdm.Dimension as a,
          sassdm.Property as b,
          sassdm.Member_Property as c,
          (
             select DISTINCT

                y.dim_internal_rk

             from

                sassdm.Dimension_Type as x,
                sassdm.Dimension as y

             where

                x.dim_type_cd='TIME' and x.dim_type_rk=y.dim_type_rk
                and (y.VALID_FROM_DTTM le datetime() le y.VALID_TO_DTTM)
                and (x.VALID_FROM_DTTM le datetime() le x.VALID_TO_DTTM)
          ) as z

       where

           b.property_cd in ('StartDate', 'EndDate', 'Level') and
          (z.dim_internal_rk=a.dim_internal_rk=c.dim_internal_rk and
           b.property_rk=c.property_rk) and
          (a.VALID_FROM_DTTM le datetime() le a.VALID_TO_DTTM) and
   /*     (b.VALID_FROM_DTTM le datetime() le b.VALID_TO_DTTM) and -- Cheetah removed */
          (c.VALID_FROM_DTTM le datetime() le c.VALID_TO_DTTM)

   ) as tempvue

where e.member_rk=tempvue.member_rk and e.dim_internal_rk=tempvue.dim_internal_rk
and (e.VALID_FROM_DTTM le datetime() le e.VALID_TO_DTTM)

group by tempvue.dimension_rk,
         tempvue.member_rk,
         e.member_cd,
         e.VALID_FROM_DTTM,
         e.VALID_TO_DTTM ;

/*+------------------------------------------------------------------------+
  | Views defined from this point forward are SPM 2.0 and SASSDM views
  | designed to help in de-normalizing the SPM 2.0 and SASSDM data to make
  | it easier for Users / Consultants to automate ETL load processing and to
  | build SPM reports using SAS. (e.g. Stored Processes or SAS/ETL Studio)
  |
  +------------------------------------------------------------------------+*/

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.METRICS_COLUMN_VIEW
  | Purpose:
  |
  | View that provides column related information. The following is contained
  | in this view:
  |
  | Name             Type Length   Format   Label
  |
  | COLUMN_ID        C     36      $36.     Column Id
  | TEMPLATE_ID      C     36      $36.     Template Id
  |
  +------------------------------------------------------------------------+*/

 create view SPMDATA.METRICS_COLUMN_VIEW as SELECT DISTINCT

    columndef.Id as COLUMN_ID label='Column Id',
    columndef.TemplateId as TEMPLATE_ID label='Template Id'


 from

    SPMDS.columndef;

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.METRICS_ELEMENT_VIEW
  | Purpose:
  |
  | View that provides a list of all the elements along with the following:
  |
  | Name              Type Length   Format   Label
  |
  | ELEMENT_ID        C     36      $36.     Element Id
  | PROJECT_ID        C     36      $36.     Project Id
  | SCORECARD_ID      C     36      $36.     Scorecard Id
  | ELEMENT_TYPE_ID   C     36      $36.     Element Type Id
  | TEMPLATE_ID       C     36      $36.     Template Id
  | PERIOD_TYPE       C     32      $32.     Period Type
  | SCOPE             C      1      $1.      Scope
  |
  +------------------------------------------------------------------------+*/
  create view SPMDATA.METRICS_ELEMENT_VIEW as SELECT DISTINCT

     element.Id as ELEMENT_ID label='Element Id',
     case
        when SCOPE = 'P' then element.containerid
        when SCOPE = 'S' then
           (
              select
                 scorecard.projectid
              from
                 SPMDS.scorecard
              where
                 element.ContainerId = scorecard.id
           )
        else ' '
     end as PROJECT_ID label='Project Id',
     case
        when SCOPE = 'P' then ' '
        when SCOPE = 'S' then element.containerid
        else ' '
     end as SCORECARD_ID label='Scorecard Id',
     element.ElementTypeId as ELEMENT_TYPE_ID label='Element Type Id',
     elementtype.TemplateId as TEMPLATE_ID label='Template Id',
     element.PeriodType as PERIOD_TYPE label='Period Type',
     elementtype.Scope as SCOPE label='Scope'

  from

     SPMDS.element left join
     SPMDS.elementtype
        on element.ElementTypeId = elementtype.Id;


/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.METRICS_SCORECARD_VIEW
  | Purpose:
  |
  | View that provides a list of all the Scorecards and their labels in
  | all the defined languages. The following is contained in this view:
  |
  | Name                 Type  Length  Format   Label
  |
  | SCORECARD_ID         C     36      $36.     Scorecard Id
  | PARENT_SCORECARD_ID  C     36      $36.     Parent Scorecard Id
  | PROJECT_ID           C     36      $36.     Project Id
  +------------------------------------------------------------------------+*/

create view SPMDATA.METRICS_SCORECARD_VIEW as select DISTINCT

   scorecard.Id as SCORECARD_ID label='Scorecard Id',
   scorecard.ParentScorecardId as PARENT_SCORECARD_ID
      label='Parent Scorecard Id',
   scorecard.ProjectId as PROJECT_ID label='Project Id'

from

    SPMDS.scorecard;


/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.METRICS_PROJECT_VIEW
  | Purpose:
  |
  | View that provides a list of all the Scorecards and their labels in
  | all the defined languages. The following is contained in this view:
  |
  | Name                 Type  Length  Format   Label
  |
  | PROJECT_ID           C     36      $36.     Project Id
  | TIME_DIMENSION_ID    C     36      $36.     Time Dimension Id
  | TIME_HIERARCHY_ID    C     36      $36.     Time Hierarchy Id
  | TEMPLATE_ID          C     36      $36.     Template Id
  +------------------------------------------------------------------------+*/

create view SPMDATA.METRICS_PROJECT_VIEW as select DISTINCT

   project.Id as PROJECT_ID label='Project Id',
   project.TimeDimensionId as TIME_DIMENSION_ID label='Time Dimension Id',
   project.TimeHierarchyId as TIME_HIERARCHY_ID label='Time Hierarchy Id',
   project.TemplateId as TEMPLATE_ID label='Template Id'

from

   SPMDS.project;


/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.METRICS_ELEMENTCOLUMN_VIEW
  | Purpose:
  |
  | View through to the elementcolumn table. The following is contained in
  | this view:
  |
  | Name               Type  Length  Format   Label
  |
  | ELEMENT_ID         C     36      $36.     Element Id
  | COLUMN_ID          C     36      $36.     Column Id
  | PERIOD_ID          N      8       11.     Period Id
  +------------------------------------------------------------------------+*/

create view SPMDATA.METRICS_ELEMENTCOLUMN_VIEW as select DISTINCT

   elementId as ELEMENT_ID label='Element Id',
   columnId as COLUMN_ID label='Column Id',
   input(periodid,11.) as PERIOD_ID label='Period Id'

from

   SPMDS.elementcolumn;

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.METRICS_TIME_VIEW
  | Purpose:
  |
  | View that contains the following information for all dimension values
  | in all dimensions of type TIME in SASSDM (Common Data Mart):
  |
  | Name               Type   Length Format        Label
  |
  | TIME_DIMENSION_ID  N      8      11.          Time Dimension Id
  | PERIOD_ID          N      8      11.          Time Period Id
  | PERIOD_CODE        C     32      $32.         Time Period Code
  | START_DATE         N      8      YYMMDDN8.    Start Date
  | END_DATE           N      8      YYMMDDN8     End Date
  | PERIOD_TYPE        C    255      $255.        Time Period Type
  | VALID_FROM_DTTM    N      8      DATETIME20.1 Valid From Datetime
  | VALID_TO_DTTM      N      8      DATETIME20.1 Valid To Datetime
  |
  +------------------------------------------------------------------------+*/
create view spmdata.METRICS_TIME_VIEW as select DISTINCT

   tempvue.dimension_rk as TIME_DIMENSION_ID label='Time Dimension Id',
   tempvue.member_rk as PERIOD_ID label='Time Period Id',
   e.member_cd as PERIOD_CODE label='Time Period Code',
   max (case
        when property_cd = 'StartDate' then
            input(property_value,yymmdd10.)
        else .
        end) as START_DATE label='Start Date' format=yymmddn8.,
   max (case
      when property_cd = 'EndDate' then
           input(property_value,yymmdd10.)
      else .
      end) as END_DATE label='End Date' format=yymmddn8.,
   max  (case
      when property_cd = 'Level' then
           property_value
      else ' '
      end ) as PERIOD_TYPE label='Time Period Type',

   e.VALID_FROM_DTTM label='Valid From Datetime',
   e.VALID_TO_DTTM label='Valid To Datetime'

from sassdm.member as e,
   (
       select DISTINCT

          c.member_rk,
          a.dimension_rk,
          a.dim_internal_rk,
          b.property_cd,
          c.property_value,
          c.valid_from_dttm,
          c.valid_to_dttm

       from

          sassdm.Dimension as a,
          sassdm.Property as b,
          sassdm.Member_Property as c,
          (
             select DISTINCT

                y.dim_internal_rk

             from

                sassdm.Dimension_Type as x,
                sassdm.Dimension as y

             where

                x.dim_type_cd='TIME' and x.dim_type_rk=y.dim_type_rk
                and (y.VALID_FROM_DTTM le datetime() le y.VALID_TO_DTTM)
                and (x.VALID_FROM_DTTM le datetime() le x.VALID_TO_DTTM)
          ) as z

       where

           b.property_cd in ('StartDate', 'EndDate', 'Level') and
          (z.dim_internal_rk=a.dim_internal_rk=c.dim_internal_rk and
           b.property_rk=c.property_rk) and
          (a.VALID_FROM_DTTM le datetime() le a.VALID_TO_DTTM) and
    /*    (b.VALID_FROM_DTTM le datetime() le b.VALID_TO_DTTM) and -- Cheetah removed */
          (c.VALID_FROM_DTTM le datetime() le c.VALID_TO_DTTM)

   ) as tempvue

where e.member_rk=tempvue.member_rk and e.dim_internal_rk=tempvue.dim_internal_rk
and (e.VALID_FROM_DTTM le datetime() le e.VALID_TO_DTTM)

group by tempvue.dimension_rk,
         tempvue.member_rk,
         e.member_cd,
         e.VALID_FROM_DTTM,
         e.VALID_TO_DTTM ;

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.COLUMN_VIEW
  | Purpose:
  |
  | View that provides column related information. The following is contained
  | in this view:
  |
  | Name             Type Length   Format   Label
  |
  | COLUMN_LABEL     C     100     $100.    Column Label
  | COLUMN_ID        C     36      $36.     Column Id
  | ELEMENT_TYPE_ID  C     36      $36.     Element Type Id
  | TEMPLATE_ID      C     36      $36.     Template Id
  |
  +------------------------------------------------------------------------+*/

 create view SPMDATA.COLUMN_VIEW as SELECT DISTINCT

    label.Label as COLUMN_LABEL label='Column Name',
    columndef.Id as COLUMN_ID label='Column Id',
    columndef.TemplateId as TEMPLATE_ID label='Template Id'


 from

    SPMDS.template,
    SPMDS.label,
    SPMDS.columndef,
    SPMDS.culture


 where

    label.Id = columndef.Id and
    template.DefaultCultureId = culture.Id and
    template.Id = columndef.TemplateId;


/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.ELEMENT_VIEW
  | Purpose:
  |
  | View that provides a list of all the elements along with the following:
  |
  | Name              Type Length   Format   Label
  |
  | ELEMENT_LABEL     C     100     $100.    Element Label
  | ELEMENT_ID        C     36      $36.     Element Id
  | PROJECT_ID        C     36      $36.     Project Id
  | SCORECARD_ID      C     36      $36.     Scorecard Id
  | ELEMENT_TYPE_ID   C     36      $36.     Element Type Id
  | TEMPLATE_ID       C     36      $36.     Template Id
  | PERIOD_TYPE       C     32      $32.     Period Type
  | SCOPE             C      1      $1.      Scope
  |
  +------------------------------------------------------------------------+*/
  create view SPMDATA.ELEMENT_VIEW as SELECT DISTINCT

     label.Label as ELEMENT_LABEL label='Element Label',
     element.Id as ELEMENT_ID label='Element Id',
     case
        when SCOPE = 'P' then element.containerid
        when SCOPE = 'S' then
           (
              select
                 scorecard.projectid
              from
                 SPMDS.scorecard
              where
                 element.ContainerId = scorecard.id
           )
        else ' '
     end as PROJECT_ID label='Project Id',
     case
        when SCOPE = 'P' then ' '
        when SCOPE = 'S' then element.containerid
        else ' '
     end as SCORECARD_ID label='Scorecard Id',
     element.ElementTypeId as ELEMENT_TYPE_ID label='Element Type Id',
     elementtype.TemplateId as TEMPLATE_ID label='Template Id',
     element.PeriodType as PERIOD_TYPE label='Period Type',
     elementtype.Scope as SCOPE label='Scope'

  from

     SPMDS.template,
     SPMDS.label,
     SPMDS.culture,
     SPMDS.element,
     SPMDS.elementtype

  where

      label.Id = element.Id and
      template.DefaultCultureId = culture.Id and
      template.Id = elementtype.TemplateId and
      element.ElementTypeId = elementtype.Id;

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.SCORECARD_VIEW
  | Purpose:
  |
  | View that provides a list of all the Scorecards and their labels in
  | all the defined languages. The following is contained in this view:
  |
  | Name                 Type  Length  Format   Label
  |
  | SCORECARD_LABEL      C     100     $100.    Scorecard Label
  | SCORECARD_ID         C     36      $36.     Scorecard Id
  | PARENT_SCORECARD_ID  C     36      $36.     Parent Scorecard Id
  | PROJECT_ID           C     36      $36.     Project Id
  +------------------------------------------------------------------------+*/

create view SPMDATA.SCORECARD_VIEW as select DISTINCT

   label.Label as SCORECARD_LABEL label='Scorecard Label',
   scorecard.Id as SCORECARD_ID label='Scorecard Id',
   scorecard.ParentScorecardId as PARENT_SCORECARD_ID
      label='Parent Scorecard Id',
   scorecard.ProjectId as PROJECT_ID label='Project Id'

from

    SPMDS.template,
    SPMDS.label,
    SPMDS.scorecard,
    SPMDS.project,
    SPMDS.culture

where

    label.Id = scorecard.Id and
    template.DefaultCultureId = culture.Id and
    scorecard.projectid = project.id and
    template.Id = project.TemplateId ;


/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.PROJECT_VIEW
  | Purpose:
  |
  | View that provides a list of all the Scorecards and their labels in
  | all the defined languages. The following is contained in this view:
  |
  | Name                 Type  Length  Format   Label
  |
  | PROJECT_LABEL        C     100     $100.    Project Label
  | PROJECT_ID           C     36      $36.     Project Id
  | TIME_DIMENSION_ID    C     36      $36.     Time Dimension Id
  | TIME_HIERARCHY_ID    C     36      $36.     Time Hierarchy Id
  | TEMPLATE_ID          C     36      $36.     Template Id
  +------------------------------------------------------------------------+*/

create view SPMDATA.PROJECT_VIEW as select DISTINCT

   label.Label as PROJECT_NAME label='Project Name',
   project.Id as PROJECT_ID label='Project Id',
   project.TimeDimensionId as TIME_DIMENSION_ID label='Time Dimension Id',
   project.TimeHierarchyId as TIME_HIERARCHY_ID label='Time Hierarchy Id',
   template.Id as TEMPLATE_ID label='Template Id'

from

   SPMDS.project,
   SPMDS.template,
   SPMDS.culture,
   SPMDS.label

where

   label.Id = project.Id and
   template.DefaultCultureId = culture.Id and
   template.Id = project.TemplateId;

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.LABEL_VIEW
  | Purpose:
  |
  | View that provides label related information. The following is contained
  | in this view:
  |
  | Name             Type Length   Format   Label
  |
  | LABEL            C     100     $100.    Label
  | ID               C     36      $36.     Id
  | CULTURE_ID       C     36      $36.     Culture Id
  |
  +------------------------------------------------------------------------+*/

 create view SPMDATA.LABEL_VIEW as SELECT

    label.Label as LABEL label='Label',
    label.Id as ID label='Id',
    label.CultureId as CULTURE_ID label='Culture Id'

 from

    SPMDS.label
;

/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.DESCRIPTION_VIEW
  | Purpose:
  |
  | View that provides label related information. The following is contained
  | in this view:
  |
  | Name             Type Length   Format   Label
  |
  | DESCRIPTION      C     100     $100.    Description
  | ID               C     36      $36.     Id
  | CULTURE_ID       C     36      $36.     Culture Id
  |
  +------------------------------------------------------------------------+*/

 create view SPMDATA.DESCRIPTION_VIEW as SELECT

    description.description as DESCRIPTION label='Description',
    description.Id as ID label='Id',
    description.CultureId as CULTURE_ID label='Culture Id'

 from

    SPMDS.description
;


/*+------------------------------------------------------------------------+
  | View Name:     SPMDATA.TIME_VIEW
  | Purpose:
  |
  | View that contains the following information for all dimension values
  | in all dimensions of type TIME in SASSDM (Common Data Mart):
  |
  | Name               Type   Length Format        Label
  |
  | TIME_DIMENSION_ID  N      8      11.          Time Dimension Id
  | PERIOD_ID          N      8      11.          Time Period Id
  | PERIOD_CODE        C     32      $32.         Time Period Code
  | START_DATE         N      8      YYMMDDN8.    Start Date
  | END_DATE           N      8      YYMMDDN8     End Date
  | PERIOD_TYPE        C    255      $255.        Time Period Type
  | VALID_FROM_DTTM    N      8      DATETIME20.1 Valid From Datetime
  | VALID_TO_DTTM      N      8      DATETIME20.1 Valid To Datetime
  |
  +------------------------------------------------------------------------+*/
create view spmdata.TIME_VIEW as select DISTINCT

   tempvue.dimension_rk as TIME_DIMENSION_ID label='Time Dimension Id',
   tempvue.member_rk as PERIOD_ID label='Time Period Id',
   e.member_cd as PERIOD_CODE label='Time Period Code',
   max (case
        when property_cd = 'StartDate' then
            input(property_value,yymmdd10.)
        else .
        end) as START_DATE label='Start Date' format=yymmddn8.,
   max (case
      when property_cd = 'EndDate' then
           input(property_value,yymmdd10.)
      else .
      end) as END_DATE label='End Date' format=yymmddn8.,
   max  (case
      when property_cd = 'Level' then
           property_value
      else ' '
      end ) as PERIOD_TYPE label='Time Period Type',

   e.VALID_FROM_DTTM label='Valid From Datetime',
   e.VALID_TO_DTTM label='Valid To Datetime'

from sassdm.member as e,
   (
       select DISTINCT

          c.member_rk,
          a.dimension_rk,
          a.dim_internal_rk,
          b.property_cd,
          c.property_value,
          c.valid_from_dttm,
          c.valid_to_dttm

       from

          sassdm.Dimension as a,
          sassdm.Property as b,
          sassdm.Member_Property as c,
          (
             select DISTINCT

                y.dim_internal_rk

             from

                sassdm.Dimension_Type as x,
                sassdm.Dimension as y

             where

                x.dim_type_cd='TIME' and x.dim_type_rk=y.dim_type_rk
                and (y.VALID_FROM_DTTM le datetime() le y.VALID_TO_DTTM)
                and (x.VALID_FROM_DTTM le datetime() le x.VALID_TO_DTTM)
          ) as z

       where

           b.property_cd in ('StartDate', 'EndDate', 'Level') and
          (z.dim_internal_rk=a.dim_internal_rk=c.dim_internal_rk and
           b.property_rk=c.property_rk) and
          (a.VALID_FROM_DTTM le datetime() le a.VALID_TO_DTTM) and
   /*     (b.VALID_FROM_DTTM le datetime() le b.VALID_TO_DTTM) and -- Cheetah removed */
          (c.VALID_FROM_DTTM le datetime() le c.VALID_TO_DTTM)

   ) as tempvue

where e.member_rk=tempvue.member_rk and e.dim_internal_rk=tempvue.dim_internal_rk
and (e.VALID_FROM_DTTM le datetime() le e.VALID_TO_DTTM)

group by tempvue.dimension_rk,
         tempvue.member_rk,
         e.member_cd,
         e.VALID_FROM_DTTM,
         e.VALID_TO_DTTM ;

  /*+----------------------------------------------------------------------+
    | REGISTER VIEWS
    |
    | Register views that are to be surfaced via ETL Studio in metadata
    | repository
    +----------------------------------------------------------------------+*/

%if "&register"="Y" %then %do;  /* Should we perform registration */


      create table SPMData_Views as
          select memname from Dictionary.tables
          where libname='SPMDATA' and
                        memname in (
          'COLUMN_VIEW',
          'DESCRIPTION_VIEW',
          'ELEMENT_VIEW',
          'LABEL_VIEW',
          'PROJECT_VIEW',
          'SCORECARD_VIEW',
          'TIME_VIEW'
          );
  quit;

  run;
  data _null_;
      set SPMData_Views;
      call execute('%regSPM(libref=SPMData, tablename=' || memname ||
                   ', ddsTree=V, ddsTreeName=Data Mart Views)' );
  run;

%end;

/*+------------------------------------------------------------------------+
  | HISTORY MIGRATION
  |
  | Create placeholder tables and register them in metadata repository
  +------------------------------------------------------------------------+*/
proc sql;

create table SPMDATA.MIGRATION_COLUMN_LOOKUP
  (
   CKEY char(256) label='1.4 Column Key',
   COLUMN char(256) label='Column Description',
   COLUMN_ID char(36) label='2.0 Column Id'
  );
create table SPMDATA.MIGRATION_ELEMENT_LOOKUP
  (
   RKEY char(256) label='1.4 Element Key',
   ROW char(256) label='Element Description',
   PERIOD_TYPE char(255) label='Period Type',
   ELEMENT_ID char(36) label='2.0 Element Id',
   tkey char(256) label='1.4 Scorecard Key'
  );
create table SPMDATA.MIGRATION_EXCEPTIONS
  (
   PROCESS_DTTM num format=DATETIME. label='Process Timestamp',
   _RECORD_ num label='Input Record',
   CAUSE char(30) label='Cause of Exception',
   table char(256) label='1.4 Scorecard',
   tkey char(256) label='1.4 Scorecard Key',
   row char(256) label='1.4 Row/Element',
   rkey char(256) label='1.4 Row/Element Key',
   column char(256) label='1.4 Column',
   ckey char(256) label='1.4 Column Key',
   value num label='Data Value',
   createTimestamp num format=DATETIME19. label='Creation Timestamp',
   fromDate num format=DATE9. label='1.4 Row/Element FROMDATE',
   toDate num format=DATE9. label='1.4 Row/Element TODATE',
   TIME_DIMENSION_ID num label='Time Dimension Id',
   PERIOD_ID num label='2.0 Period Id',
   PERIOD_CODE char(32) label='2.0 Time Period Description',
   START_DATE num format=DATE9. label='2.0 Time Period Start Date',
   END_DATE num format=DATE9. label='2.0 Time Period End Date',
   PERIOD_TYPE char(255) label='2.0 Time Period Type',
   VALID_FROM_DTTM num format=DATETIME20.1 informat=DATETIME20.1
                       label='2.0 Time Period Valid From Datetime',
   VALID_TO_DTTM num format=DATETIME20.1 informat=DATETIME20.1
                     label='2.0 Time Period Valid To Datetime',
   ABSOLUTE_DIFF num label='Absolute Difference'
  );
create table SPMDATA.MIGRATION_EXCEPTIONS_SUMMARY
  (
   PROCESS_DTTM num format=DATETIME. label='Process Timestamp',
   CAUSE char(30) label='Cause of Exception',
   COUNT num label='Exception Count'
  );
create table SPMDATA.MIGRATION_PERIODTYPE
  (
   ELEMENT_ID char(36) label='2.0 Element Id',
   PERIOD_TYPE char(255) label='2.0 Time Period Type'
  );
create table SPMDATA.MIGRATION_STATUS
  (
   PROCESS_DTTM num format=DATETIME. label='Process Timestamp',
   USER_ID char(30) label='User Id',
   ENVKEY char(256) label='1.4 Environment Key',
   STAGED_RECORDS num label='Valid Records',
   EXCEPTION_RECORDS num label='Exception Records',
   ELEMENT_UPDATES num label='Elements updated with Period Type',
   JOB_RC num label='Maximum Return Code'
  );
create table SPMDATA.MIGRATION_TIME_LOOKUP
  (
   fromDate num format=DATE9.,
   toDate num format=DATE9.,
   DATE_RANGE num label='Date Range Counter',
   TIME_DIMENSION_ID num label='Time Dimension Id',
   PERIOD_ID num label='2.0 Period Id',
   PERIOD_CODE char(32) label='2.0 Time Period Description',
   START_DATE num format=DATE9. label='2.0 Time Period Start Date',
   END_DATE num format=DATE9. label='2.0 Time Period End Date',
   PERIOD_TYPE char(255) label='2.0 Time Period Type',
   VALID_FROM_DTTM num format=DATETIME20.1 informat=DATETIME20.1
                       label='2.0 Time Period Valid From Datetime',
   VALID_TO_DTTM num format=DATETIME20.1 informat=DATETIME20.1
                       label='2.0 Time Period Valid To Datetime',
   ABSOLUTE_DIFF num label='Absolute Difference'
  );

create table SPMDATA.MIGRATION_VALIDATED
  (
   _RECORD_ num label='Input Record',
   ELEMENT_ID char(36) label='2.0 Element Id',
   COLUMN_ID char(36) label='2.0 Column Id',
   PERIOD_ID num label='2.0 Period Id',
   VALUE num label='Data Value',
   PROCESS_DTTM num format=DATETIME. label='Process Timestamp',
   table char(256) label='1.4 Scorecard',
   tkey char(256) label='1.4 Scorecard Key',
   row char(256) label='1.4 Row/Element',
   rkey char(256) label='1.4 Row/Element Key',
   column char(256) label='1.4 Column',
   ckey char(256) label='1.4 Column Key',
   createTimestamp num format=DATETIME19. label='Creation Timestamp',
   fromDate num format=DATE9. label='1.4 Row/Element FROMDATE',
   toDate num format=DATE9. label='1.4 Row/Element TODATE',
   TIME_DIMENSION_ID num label='Time Dimension Id',
   PERIOD_CODE char(32) label='2.0 Time Period Description',
   START_DATE num format=DATE9. label='2.0 Time Period Start Date',
   END_DATE num format=DATE9. label='2.0 Time Period End Date',
   PERIOD_TYPE char(255) label='2.0 Time Period Type',
   VALID_FROM_DTTM num format=DATETIME20.1 informat=DATETIME20.1
                       label='2.0 Time Period Valid From Datetime',
   VALID_TO_DTTM num format=DATETIME20.1 informat=DATETIME20.1
                     label='2.0 Time Period Valid To Datetime',
   ABSOLUTE_DIFF num label='Absolute Difference'
  );
create table SPMDATA.SPM_14_HISTORY
  (
   table char(256),
   tkey char(256),
   row char(256),
   rkey char(256),
   column char(256),
   ckey char(256),
   view char(256),
   vkey char(256),
   value num,
   createTimestamp num format=DATETIME19.,
   fromDate num format=DATE9.,
   toDate num format=DATE9.
  );

%if "&register"="Y" %then %do;  /* Should we perform registration */
  /*+----------------------------------------------------------------------+
    | Register MIGRATION tables
    +----------------------------------------------------------------------+*/
  create table SPMData_Migration_Tables as
          select memname from Dictionary.tables
          where libname='SPMDATA' and
                        memname in (
          'MIGRATION_EXCEPTIONS',
          'MIGRATION_EXCEPTIONS_SUMMARY',
          'MIGRATION_STATUS',
          'MIGRATION_VALIDATED',
          'SPM_14_HISTORY'
          );
  quit;

  data _null_;
      set SPMData_Migration_Tables;
      if memname='SPM_14_HISTORY' then
        call execute('%regSPM(libref=SPMData, tablename=' || memname ||
                   ', ddsTree=S, ddsTreeName=History Migration Tables)' );
      else
        call execute('%regSPM(libref=SPMData, tablename=' || memname ||
                   ', ddsTree=T, ddsTreeName=History Migration Tables)' );
  run;
%end;

/*+------------------------------------------------------------------------+
  | METRICS DATA LOAD
  |
  | Create placeholder tables and register them in metadata repository
  +------------------------------------------------------------------------+*/

proc sql;
create table SPMDATA.METRICS_EXCEPTIONS
  (
   PROCESS_DTTM num format=DATETIME. label='Process Timestamp',
   _RECORD_ num label='Input Record',
   CAUSE char(40) label='Cause of Exception',
   ELEMENT_ID char(36) label='Element Id',
   COLUMN_ID char(36) label='Column Id',
   PERIOD_ID num label='Period Id',
   VALUE num label='Data Value',
   PROJECT_ID char(36) label='Project Id',
   E_TEMPLATE_ID char(36) label='Element Template Id',
   C_TEMPLATE_ID char(36) label='Column Template Id',
   E_PERIOD_TYPE char(255) label='Element Period Type',
   C_PERIOD_TYPE char(255) label='Time Period Type',
   TIME_DIMENSION_ID num label='Time Dimension Id'
  );
create table SPMDATA.METRICS_EXCEPTIONS_SUMMARY
  (
   PROCESS_DTTM num format=DATETIME. label='Process Timestamp',
   CAUSE char(40) label='Cause of Exception',
   COUNT num label='Exception Count'
  );
create table SPMDATA.METRICS_STATUS
  (
   PROCESS_DTTM num format=DATETIME. label='Process Timestamp',
   USER_ID char(30) label='User Id',
   STAGED_RECORDS num label='Valid Records',
   UPDATED num label='# Updated',
   INSERTED num label='# Inserted',
   EXCEPTION_RECORDS num label='Exception Records',
   JOB_RC num label='Maximum Return Code'
  );
create table SPMDATA.METRICS_INPUT
  (
   ELEMENT_ID char(36) label='Element Id',
   COLUMN_ID char(36) label='Column Id',
   PERIOD_ID num label='Period Id',
   VALUE num label='Data Value'
  );

%if "&register"="Y" %then %do; /* Should we perform registration */
  /*+----------------------------------------------------------------------+
    | Register METRICS tables
    +----------------------------------------------------------------------+*/

  create table SPMData_Metrics_Tables as
          select memname from Dictionary.tables
          where libname='SPMDATA' and
                        memname in (
          'METRICS_EXCEPTIONS',
          'METRICS_EXCEPTIONS_SUMMARY',
          'METRICS_STATUS',
          'METRICS_INPUT'
          );
  quit;

  data _null_;
      set SPMData_Metrics_Tables;
          if memname='METRICS_INPUT' then
        call execute('%regSPM(libref=SPMData, tablename=' || memname ||
                   ', ddsTree=S, ddsTreeName=Data Mart Load Tables)' );
      else
            call execute('%regSPM(libref=SPMData, tablename=' || memname ||
                   ', ddsTree=T, ddsTreeName=Data Mart Load Tables)' );
  run;
%end;

%mend spmetl;
