/*--------------------------------------------------------------
 *  Copyright (c) 2003 by SAS Institute Inc., Cary NC USA
 *
 *  NAME:      regRMet.sas
 *
 *  PURPOSE:   Create and register metric table.
 *
 *  USAGE:
 *
 *  metric_databaseSchema      - Name of OMR DatabaseSchema object associated to physicalTable object
 *  metric_sasLibrary          - Name of OMR SASLibrary object associated to physicalTable object
 *  metric_physicalTable       - Name of PhysicalTable to be registered
 *  metric_descPhysicalTable   - Description of PhysicalTable to be registered
 *  metric_metarepository      - Name of the metadata repository to be used
 *  metric_dim_rk              - Dim column definition. Each column definition separated by a space.
 *  metric_value_rk            - Value column definition. Each column definition separated by a space.
 *  metric_fixed_columns       - Fixed column definition. Each column definition separated by a space.
 *
 * All column definitions must have the following syntax:
 *       COLNAME:TYPE<(SIZE)<:SUMMARYROLE<:DIMENSION_RK:HIERARCHY_RK>>>
 *
 * EXAMPLE:

  %regrmet
    (metric_databaseSchema=&SOLUTIONS_DB_SCHEMA,
     metric_sasLibrary=SASSDM,
     metric_physicalTable=METRIC,
     metric_descPhysicalTable=METRIC Description,
     metric_metarepository=&SlnRepos,
     metric_dim_rk=col1:int:ID:1:1 col2:char(50):ID,
     metric_value_rk=val1:int(10):STATISTIC val2:int(10):STATISTIC
     );
*/

%macro regrmet
  (metric_databaseSchema=,
   metric_sasLibrary=SASSDM,
   metric_physicalTable=,
   metric_descPhysicalTable=*NONE*,
   metric_metarepository=&SlnRepos,
   metric_dim_rk=,
   metric_value_rk=
   );

%let metric_fixed_columns=MEASURE_RK:int:ID DIRECTIVE_TXT:char(255) MODIFIED_DT:datetime SOURCE_SYSTEM_RK:int;
%let metric_summaryRoleDs=WORK._sumRole%sysfunc(compress(%sysfunc(datetime()), '.'));

data _null_;
  if symgetc('metric_descPhysicalTable') eq '*NONE*' then
    call symput('metric_descPhysicalTable', symget('metric_physicalTable'));
run;

filename _tmpfile temp;
data &metric_summaryRoleDs(keep=colname--hierarchy_rk);
  length
        colname      $32
        summaryRole  $10
        dimension_rk $20
        hierarchy_rk $20
  ;
  file _tmpfile;
  array _dim(3) $30 ("metric_fixed_columns"
                     "metric_dim_rk" "metric_value_rk" );

  line = 'execute (DROP TABLE if EXISTS '
             !! symget('metric_physicalTable') !! ') by PASSTHRU;' ;
  put line;
  put 'execute (';
  line = 'create table ' !! symget('metric_physicalTable') ;
  put line;
  do _dimarray = 1 to dim(_dim);
     if _dimarray eq 1 then
        put '(';

     do i = 1 to 99;
        _tmpword = scan(symget(_dim(_dimarray)), i, '');
        if _tmpword eq '' then leave;

        colname = scan(_tmpword, 1, ':');
        _tmpparm2 = scan(_tmpword, 2, ':');
        summaryRole  = scan(_tmpword, 3, ':');
        dimension_rk = scan(_tmpword, 4, ':');
        hierarchy_rk = scan(_tmpword, 5, ':');

        if summaryRole ne '' then
          output;

        if colname eq 'DIRECTIVE_TXT' or summaryRole='STATISTIC' then
          put @5 colname @40 _tmpparm2 ',';
        else
          put @5 colname @40 _tmpparm2 'NOT NULL ,';


     end;

     if _dimarray eq dim(_dim) then do;
       put @5 "UNIQUE KEY UNIQUE_KEY (MEASURE_RK" @;
       do i = 1 to 15;
          _tmpword = scan(symget('metric_dim_rk'), i, '');
          if _tmpword eq '' then leave;

          colname = scan(_tmpword, 1, ':');
          put ', ' colname @;
       end;
       put '),';
       put @5 "KEY MEASURE_RK (MEASURE_RK),";
       put @5 "KEY MODIFIED_DT (MODIFIED_DT)";

       line = ') COMMENT= ' !! quote(strip(symget('metric_descPhysicalTable')))
                            !! ' TYPE = InnoDB ) by PASSTHRU;';
       put line;
     end;
  end;
run;

options metarepository=&metric_metarepository;

%LibConOp(sesetl_sasLibRef=&metric_sasLibrary,
          sesetl_ChkDepRepos=Y
          );

proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
           ( &sesetl_BestLogin &sesetl_ConnectionOptions );

  %include _tmpfile;

  disconnect from PASSTHRU ;
quit;

filename _tmpfile;

%regrdata(
  sesetl_databaseSchema=&metric_databaseSchema,
  sesetl_sasLibrary=&metric_sasLibrary,
  sesetl_physicalTable=&metric_physicalTable,
  sesetl_altETLStudioTreeNm=SAS Data Integration Studio Custom Tree,
  sesetl_altEtlStudioGroupNm=Data Integration Studio Groups,
  sesetl_Tree=M,
  sesetl_TreeName=Solution Data Marts (SDM),
  sesetl_ValidTreeTypes=D:Dimension Tables @ R:Reference Tables @ F:Fact Tables @ M:Metric Tables,
  sesetl_propertySetNm=DBMSNames,
  sesetl_ChkDepRepos=Y,
  sesetl_OMRTableObjName=&metric_physicalTable,
  sesetl_summaryRoleDs=&metric_summaryRoleDs
   );

%mend regrmet;
